#!/usr/bin/env python
#
#       .ga3 and .cmbl extractor
#       by Luke Bryan, July 2009
#       This program tries to open Vernier data files.
#       
#       This program is free software; you can redistribute it and/or modify
#       it under the terms of the GNU General Public License as published by
#       the Free Software Foundation; either version 2 of the License, or
#       (at your option) any later version.
#       
#       This program is distributed in the hope that it will be useful,
#       but WITHOUT ANY WARRANTY; without even the implied warranty of
#       MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#       GNU General Public License for more details.

from xml.dom.minidom import parseString
import sys
import os

#GUI:
import Tkinter as tk
import tkFileDialog
import tkMessageBox


def main():
	args = sys.argv[1:]
	
	print args
	if (len(args) >= 1 and args[0]):
		if (args[0]=="-h" or args[0]=="--help"):
			print """This program converts Vernier data files to .csv spreadsheets.

-------Instructions-------
To view .ga3 or .cmbl file data:
ga3extract.py /path/to/file.ga3

To extract data to spreadsheet file:
ga3extract.py /path/to/file.ga3 /path/to/output.csv

To convert a file using GUI filepickers, run:
ga3extract.py
"""
			return 0
	else:
		#No command line option, use GUI:
		print "Opening file dialogs"
		print "To run this program at command line, try ga3extract.py -h"
		root = tk.Tk()
		root.withdraw()
		inp = tkFileDialog.askopenfilename(title="Select a file to convert",filetypes=[("Vernier data files",("*.ga3","*.cmbl","*.xmbl","*.gmbl","*.qmbl"))])
		if (inp != ""):
			out = tkFileDialog.asksaveasfilename(title="Select where to save the spreadsheet output",filetypes=[(".csv spreadsheet","*.csv")])
			if (out != ""):
				args=[inp,out]
			else:
				return 0 #cancelled
		else:
			return 0 #cancelled

	out = "" # for converted output

	print "Opening file..."
	file = open(args[0])
	try:
		contents = file.read().replace("&","&amp;") # these files most often don't have correct xml, fix it.
		dom = parseString(contents)
	except:
		print "Error. Not a valid .xml file."
		err=os.system('zenity --error --text="Error! Not a valid xml file.\nPlease choose valid xml data file."')
		if (err): #command not found.
			root = tk.Tk()
			root.withdraw()
			tkMessageBox.showerror("Error","Error! Not a valid xml file.\nPlease choose valid xml data file.")
		return 1
	
	texts = dom.getElementsByTagName('TextText')
	for el in texts:
		out += "\""+fixtext(textContent(el.childNodes))+"\"\n"
		# The text in csv text format. (In quotes, " turned to "", &#num; corrected.)
		
	functions = dom.getElementsByTagName('FunctionModel')
	if len(functions)>0:
		out += "---Functions---\n"
		for function in functions:
			for el in function.childNodes:
				if (el.nodeName == "DataObjectName"):
					out+="Name:\n\""+fixtext(textContent(el.childNodes))+"\"\n"
				elif (el.nodeName == "FunctionModelString"):
					out+="Function:\n"+textContent(el.childNodes)+"\n"
				elif (el.nodeName == "FunctionCoefficientArray"):
					coeff = textContent(el.childNodes).split()
					coeff.pop(0) #First number is count of the coefficients.
					out+="Coefficient Numbers:," + ",".join(coeff) + "\n"
				elif (el.nodeName == "FunctionCoefficientUncertaintyArray"):
					coeff = textContent(el.childNodes).split()
					coeff.pop(0) #First number is count of the coefficients.
					out+="Coefficient Uncertainty:," + ",".join(coeff) + "\n"
			out+="\n"
	out+="---Data---\n"


	columns = dom.getElementsByTagName('DataColumn')
	columnArr = [['Data Table: -->']] #first column of data table
	for col in columns:
		coltext = "" #this one column
		for el in col.childNodes:
			if (el.nodeName == "DataObjectName"):
				coltext += fixtext(textContent(el.childNodes)) # Column label
			elif (el.nodeName == "ColumnUnits"):
				if (textContent(el.childNodes) != ''):
					coltext += " ("+fixtext(textContent(el.childNodes))+")" # Units
			elif (el.nodeName == "ColumnCells"):
				txt = textContent(el.childNodes) # Data
				if (txt.find("\nZ")>-1):
					txt=txt[:txt.find("\nZ")] # trim the Z1 Z1 Z2 Z2 etc.
				coltext += txt
		columnArr.append(coltext.split("\n"))
	#print columnArr

	if (len(args)>=2 and args[1]):
		print "Converting and saving to file..."
		# add csv columns.
		max=0;
		for col in columnArr:
			if (len(col)>max):
				max=len(col)
		
		#put columns together in csv format:
		for i in range(max):
			out+="\n"
			for col in columnArr:
				if (i<len(col)):
					out += col[i]
				out += ","
		
		#Write data file
		try:
			file = open(args[1],"w")
			file.write(out.encode("utf-8"))
			print "Successfully saved to "+args[1]+"."
		except:
			print "Couldn't save to "+arg[1]+"."
			err=os.system('zenity --error --text="Error! Couldn\'t write to file."')
			if (err):
				root = tk.Tk()
				root.withdraw()
				tkMessageBox.showerror("Error","Error! Couldn't write to file.")
			return 1
	else:
		#Show data
		print out
		columnArr.pop(0) # - spacer column
		for col in columnArr:
			print "\n".join(col)
			print "\n"
	return 0

def textContent(nodes):
	out = ""
	for node in nodes:
		if (node.nodeType == node.TEXT_NODE):
			out += node.data
	return out.replace("&amp;","&")


def fixtext(str): # Fix the &#num; characters, turn " into ""
	#print str
	fixstr = fixentity(str)
	while (fixstr != str):
		str=fixstr
		fixstr=fixentity(str)
		#try to fix it again.
	#print fixstr
	return fixstr.replace("\"","\"\"")

def fixentity(str):
	ent = str.find("&#")
	if (ent > -1 and str.find(";",ent)>ent):
		end = str.find(";",ent+2)
		try:
			return str[0:ent] + unichr(int(str[ent+2:end])) + str[end+1:]
		except:
			return str
	else:
		return str

try:
	if __name__ == '__main__': main()
except IOError:
	root = tk.Tk()
	root.withdraw()
	print "File error, bad filename."
	tkMessageBox.showerror("Error","File error, bad filename.")
